#!/usr/bin/env python3
#

import os
import re
import os.path
import shutil
import subprocess
import plistlib

# Config
bundleId = "app.feedjournaler.FeedJournaler"

ascUsername = "lmika314@gmail.com"
ascPassword = "ahgk-tpaq-nhyl-bmjv"
ascProvider = "7D4333Y58H"

buildDir = "/Users/leonmika/src/tools/day-one-sync/build"

# Derived
archivePath = os.path.join(buildDir, "Feed Journaler.xcarchive")
exportOptions = os.path.join(buildDir, "exportOptions.plist")
exportArchivePath = os.path.join(buildDir, "Feed Journaler.archive")
exportArchiveAppPath = os.path.join(buildDir, "Feed Journaler.app")
zipPath = os.path.join(buildDir, "Feed Journaler.zip")

def run_command(cmd, **kwds):
    print("CMD: ", cmd)
    return subprocess.run(cmd, **kwds)
    

def build_archive():
    # Clean the build directory
    shutil.rmtree(buildDir)
    os.mkdir(buildDir)

    # Build the archive
    run_command(["xcodebuild", "archive",
        "-workspace", "Feed Journaler.xcworkspace", "-scheme", "Feed Journaler", "-configuration", "Release",
        "-archivePath", archivePath])

def export_archive():
    # Generate the export options
    #
    # Certificate IDs, found using 'security find-identity -v'
    #
    with open(exportOptions, "wb") as f:
        plistlib.dump({
            "destination": "export",
            "method": "developer-id",
            "provisioningProfiles": {
                bundleId: "Feed Journaler v0.3",       # Provisioning profile name
            }
        }, f)

    # Export the archive
    run_command(["xcodebuild", "archive", "-archivePath", archivePath,
        "-exportArchive", "-exportOptionsPlist", exportOptions, "-exportPath", exportArchivePath])


def submit_archive_for_noterization():
    # TEMP
    os.rename(os.path.join(exportArchivePath, "Feed Journaler.app"), exportArchiveAppPath),

    # Use ditto to zip the archive
    #
    # See https://blog.frostwire.com/2019/08/27/apple-notarization-the-signature-of-the-binary-is-invalid-one-other-reason-not-explained-in-apple-developer-documentation/
    run_command(["ditto", "-c", "-k", "--sequesterRsrc", "--keepParent", exportArchiveAppPath, zipPath])
    
    # Upload to Apple
    #
    # SAMPLE OUTPUT
    #    No errors uploading '/Users/leonmika/src/tools/day-one-sync/build/Feed Journaler.zip'.
    #RequestUUID = 03b8e55b-ece6-4b4b-b275-daddc1bf4ae6
    idRegex = re.compile('RequestUUID = ([0-9a-f-]+)')
    cmdOut = run_command(["xcrun", "altool", "--notarize-app", "--file", zipPath, "--primary-bundle-id", bundleId,
        "--username", ascUsername, "--password", ascPassword, "--asc-provider", ascProvider], capture_output=True)
        
    idMatch = idRegex.search(cmdOut.stdout)
    if idMatch is None:
        raise Exception("cannot find request ID")
        
    requestId = idMatch.group(1)

def check_noterization_status(request_id):

    # SAMPLE OUTPUT
    #          Date: 2022-04-27 20:05:31 +0000
    #          Hash: 6ea2458b1816637de20bdce7f12d6df5f6a8d244c8b3a90b05062c298641f43c
    #    LogFileURL: https://osxapps-ssl.itunes.apple.com/itunes-assets/Enigma122/v4/18/98/1c/18981c3f-4f04-219b-66c8-f7cc8bc60094/developer_log.json?accessKey=1651284442_8282355759514148296_cKtIAjbNntK2tI5HzIORv%2BLuuVw99Yfmgrs0H%2Fzi15ObyA1bIaQEjYem5Xp9tnBoiuqJuX175tT9HcE7%2BLRJSWSp24cNVh%2BHCgfsJ2eG9J9JnpyCwAailwWU6kpd0JZM5bBBznozrWpiPfYJpOkdOKbqp0Y6Eyaqr3v8GnydKMM%3D
    #   RequestUUID: 03b8e55b-ece6-4b4b-b275-daddc1bf4ae6
    #        Status: success
    #   Status Code: 0
    #Status Message: Package Approved

    run_command(["xcrun", "altool", "--notarization-info", request_id,
        "--username", ascUsername, "--password", ascPassword, "--asc-provider", ascProvider])


#build_archive()
#export_archive()
#submit_archive_for_noterization()

check_noterization_status("a56c4bda-f143-4359-9b33-ddfaa8e10774")



# FILENAME TO TEST = s3://lmika-public-files/Apps/FeedJournaler/NotarizationTest/Feed Journaler.zip
